/*
 *   This file is part of Dianara
 *   Copyright 2012  JanKusanagi <janjabber@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .
 */

#include "post.h"

Post::Post(DiasporaController *diasporaController,
           QMap<QString,QString> postData,
           QWidget *parent)  :  QFrame(parent)
{
    this->setFrameStyle(QFrame::Box | QFrame::Raised);
    //this->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken); // alternative?

    this->setSizePolicy(QSizePolicy::MinimumExpanding,
                        QSizePolicy::Maximum);

    leftColumnLayout = new QVBoxLayout();
    leftColumnLayout->setAlignment(Qt::AlignLeft | Qt::AlignTop);


    /////////////////////////////////////////////////// Left column, post Meta info


    // Author avatar
    postAuthorAvatar = new QLabel();

    // Get local file name, which is stored in base64 hash form
    QString avatarFilename = MiscHelpers::getCachedAvatarFilename(postData.value("authorAvatar"));

    //qDebug() << "Avatar remote file:" << postData.value("authorAvatar");
    //qDebug() << "Avatar local file:" << avatarFilename;

    QFile avatarFile(avatarFilename);
    if (avatarFile.exists(avatarFilename))
    {
        // Load avatar if already cached
        postAuthorAvatar->setPixmap(QPixmap(avatarFilename)
                                    .scaled(48, 48,
                                            Qt::KeepAspectRatio,
                                            Qt::SmoothTransformation));
        qDebug() << "Using cached avatar";
    }
    else
    {
        // Placeholder image
        postAuthorAvatar->setPixmap(QIcon::fromTheme("user-identity").pixmap(48,48));
        qDebug() << "Using placeholder";

        // Download avatar for next time
        diasporaController->getAvatar(postData.value("authorAvatar"));
    }

    leftColumnLayout->addWidget(postAuthorAvatar);


    QFont authorFont;
    authorFont.setBold(true);
    authorFont.setUnderline(true);

    // Author name
    postAuthorName = new QLabel(postData.value("author"));
    postAuthorName->setTextFormat(Qt::PlainText);
    postAuthorName->setWordWrap(true);
    postAuthorName->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    postAuthorName->setFont(authorFont);

    leftColumnLayout->addWidget(postAuthorName);


    QFont detailsFont;
    detailsFont.setPointSize(detailsFont.pointSize() - 2); // FIXME:check size first

    // Post creation time
    QString createdAt = postData.value("createdAt");
    // Format is "Combined date and time in UTC", like "2012-02-07T01:32:02Z"
    // As specified in ISO 8601 http://en.wikipedia.org/wiki/ISO_8601
    QString exactPostTime = Timestamp::localTimeDate(createdAt);
    createdAt = Timestamp::fuzzyTime(createdAt);

    postCreatedAt = new QLabel(createdAt + "\n");
    postCreatedAt->setToolTip(exactPostTime);
    postCreatedAt->setTextFormat(Qt::PlainText);
    postCreatedAt->setWordWrap(true);
    postCreatedAt->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    postCreatedAt->setFont(detailsFont);
    leftColumnLayout->addWidget(postCreatedAt);


    // Post type: StatusMessage or Reshare
    if (postData.value("postType") == "Reshare")
    {
        postType = new QLabel(tr("Reshare of a post by:"));
        postType->setWordWrap(true);
        postType->setAlignment(Qt::AlignLeft | Qt::AlignTop);
        postType->setFont(detailsFont);
        leftColumnLayout->addWidget(postType);

        postRootAuthor = new QLabel(postData.value("rootAuthor"));
        postRootAuthor->setTextFormat(Qt::PlainText);
        postRootAuthor->setWordWrap(true);
        postRootAuthor->setAlignment(Qt::AlignLeft | Qt::AlignTop);
        postRootAuthor->setFont(detailsFont);
        leftColumnLayout->addWidget(postRootAuthor);
    }


    QString likesCount = postData.value("likesCount");
    if (likesCount != "0")
    {
        postLikesCount = new QLabel(QString::fromUtf8("\342\231\245 ")
                                  + tr("%1 likes").arg(likesCount));
        postLikesCount->setWordWrap(true);
        postLikesCount->setAlignment(Qt::AlignLeft | Qt::AlignTop);
        postLikesCount->setFont(detailsFont);
        leftColumnLayout->addWidget(postLikesCount);
    }

    QString commentsCount = postData.value("commentsCount");
    if (commentsCount != "0")
    {
        postCommentsCount = new QLabel(tr("%1 comments").arg(commentsCount));
        postCommentsCount->setWordWrap(true);
        postCommentsCount->setAlignment(Qt::AlignLeft | Qt::AlignTop);
        postCommentsCount->setFont(detailsFont);
        leftColumnLayout->addWidget(postCommentsCount);
    }

    QString resharesCount = postData.value("resharesCount");
    if (resharesCount != "0")
    {
        postResharesCount = new QLabel(QString::fromUtf8("\342\231\272 ")
                                     + tr("%1 reshares").arg(resharesCount));
        postResharesCount->setWordWrap(true);
        postResharesCount->setAlignment(Qt::AlignLeft | Qt::AlignTop);
        postResharesCount->setFont(detailsFont);
        leftColumnLayout->addWidget(postResharesCount);
    }



    ////////////////////////////////////////////////// Right column, content
    ///////////////////////// Post text

    postText = new QTextBrowser();
    postText->setAlignment(Qt::AlignLeft | Qt::AlignTop);

    postText->setWordWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
    postText->setOpenExternalLinks(true); // don't open HTTP links, open in browser
    //postText->setWordWrapMode(QTextOption::NoWrap);
    //postText->setWordWrapMode(QTextOption::WordWrap);

    postText->setReadOnly(true); // it's default with QTextBrowser, but still...
    postText->setPlainText(postData.value("text")); // Specifically plain text,
                                                    // DON'T autoparse HTML!

    // Get list of images URL's
    imageList = Markdown::imageList(postText->toHtml());
    qDebug() << "Image List:" << imageList;
    foreach (QString imageURL, imageList)
    {
        diasporaController->getImage(imageURL);
    }


    // Then, with the filtered HTML, convert only the Markdown codes to HTML
    postText->setHtml(Markdown::toHtml(postText->toHtml()));


    int postLines = postText->document()->lineCount();
    qDebug() << "Post lines:" << postLines;

    int postMinimumHeight = postLines * 3 * postText->fontInfo().pixelSize();
    if (postMinimumHeight > 400)  // Don't allow a post to be higher than 400 pix
    {
        postMinimumHeight = 400;  // Scrollbars will appear
        qDebug() << "Post would require more space: scrollbars might appear";
    }
    qDebug() << "Post minimum height:" << postMinimumHeight;
    postText->setMinimumHeight(postMinimumHeight);


    mainLayout = new QHBoxLayout();

    mainLayout->addLayout(leftColumnLayout, 1); // stretch 1/5
    mainLayout->addWidget(postText,         4); // stretch 4/5

    mainLayout->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    this->setLayout(mainLayout);

    qDebug() << "Post created";
}



Post::~Post()
{
    qDebug() << "Post destroyed";
}


void Post::resizeEvent(QResizeEvent *event)
{
    //postText->adjustSize();
    //postText->resize(postText->contentsRect().size());
    //postText->resize(postText->frameSize());

    //this->adjustSize();

    event->accept();
}
